@@ -18,25 +18,25 @@ sub lngbase {
  ,-nup=>['',     '襫騩 ஢']
  ,-nth=>['',    ' ਫ -㬮砭']
  ,-bck=>['',    '  ।饬 ࠭']
- ,-lgn=>['',    '  ⥬']
- ,-lst=>['',     '  ᮣ᭮ ᫮ 롮ન (᪠)']
- ,-lsr=>['',     ' ᯨ᮪ ᥩ']
- ,-qry=>['',   '᫮ 롮ન (᪠) ᥩ']
- ,-crt=>['',  '  , ⮡ ⥬ \'⠢\'    ']
- ,-sel=>['',  '()    ,  ']
+ ,-lgn=>['',    '  ⥬, ॣ஢']
+ ,-lst=>['',  '() ।⠢,   ᮣ᭮ ᫮ 롮ન (᪠)']
+ ,-lsr=>['롮ઠ',  ' ।⠢']
+ ,-qry=>['',   ' ᫮ 롮ન (᪠) ᥩ']
+ ,-crt=>['',  '  ᮧ  , ⮡ ⥬ \'⠢\'    ']
+ ,-sel=>['',  '()    ; ३  ।஢   ᬮ  ३ १⮢ ।஢']
  ,-prn=>['',   'ଠ  ⠭']
- ,-edt=>['',   '४⥫ ᬮ / ।஢ ']
- ,-frm=>['',     '१㧨  ᬮ ,  ']
- ,-upd=>['࠭','     , ࠭  ⮩ ']
- ,-del=>['',  '     ']
- ,-ins=>['⠢', '⠢     ']
+ ,-edt=>['',   ' ० ।஢ () ']
+ ,-frm=>['',     '१㧨   ࢥ, ॢ᫨ ']
+ ,-upd=>['࠭','࠭ १ ।஢ ()    ']
+ ,-del=>['',  '    ']
+ ,-ins=>['⠢', ' १ ।஢      ']
  ,-hlp=>['ࠢ',  ' ࠭ ࠢ']
 
  ,'op!let' =>['',    '⢨ \'$_\' 饭']
  ,'!constr'=>['',    '襭 ࠭祭']
 
  ,'Help'      =>['ࠢ',         '']
- ,'Lists'     =>['।⠢ (᪨)','।⠢, ᪨   ᥩ, ।  ਫ']
+ ,'Lists'     =>['।⠢','।⠢, ।  ਫ']
  ,'Fields'    =>['',            ' , ।  ਫ']
  ,'Commands'  =>['',         ',   ᯮ짮']
  );
