@@ -19,7 +19,11 @@ sub lngbase {
  ,'ddlbclose'    =>['x',     '']
  ,'ddlbsetvalue' =>['<',     ' ']
  ,'Files'        =>['', ' ']
- ,'+|-'          =>['+/-',   '  ']
+ ,'+|-'          =>['+/-',   ' /  / ']
+ ,'fsopens'	 =>['...',   ' ']
+ ,'fsclose'	 =>['','  ']
+ ,'fsbrowse'	 =>['','  ']
+ ,'fsdelmrk'	 =>['','  ']
  );
  @msg
 }
